"""Built-in trading strategies for Fintics.
"""

from .AoStrategy import AoStrategy
from .ApoCrossoverStrategy import ApoCrossoverStrategy
from .AroonCrossoverStrategy import AroonCrossoverStrategy
from .AroonStrategy import AroonStrategy
from .AtrChangeStrategy import AtrChangeStrategy
from .AtrCrossoverStrategy import AtrCrossoverStrategy
from .BollingerBandsStrategy import BollingerBandsStrategy
from .BopStrategy import BopStrategy
from .CciStrategy import CciStrategy
from .CmoStrategy import CmoStrategy
from .CorrelationCrossoverStrategy import CorrelationCrossoverStrategy
from .CorrelationMaCrossoverStrategy import CorrelationMaCrossoverStrategy
from .DmiStrategy import DmiStrategy
from .GarmanKlassStrategy import GarmanKlassStrategy
from .HeikinAshiStrategy import HeikinAshiStrategy
from .HistoricalVolatilityStrategy import HistoricalVolatilityStrategy
from .IchimokuCloudStrategy import IchimokuCloudStrategy
from .KalmanFilterChangeStrategy import KalmanFilterChangeStrategy
from .KalmanFilterCrossoverStrategy import KalmanFilterCrossoverStrategy
from .MacdCustomStrategy import MacdCustomStrategy
from .MacdStrategy import MacdStrategy
from .MamaStrategy import MamaStrategy
from .MfiStrategy import MfiStrategy
from .MidpointStrategy import MidpointStrategy
from .MlCrossoverStrategy import MlCrossoverStrategy
from .MlStrategy import MlStrategy
from .MomentumCrossoverStrategy import MomentumCrossoverStrategy
from .MomentumStrategy import MomentumStrategy
from .MovingAverageCrossoverStrategy import MovingAverageCrossoverStrategy
from .MovingAverageDifferenceStrategy import MovingAverageDifferenceStrategy
from .ParabolicSarStrategy import ParabolicSarStrategy
from .PpoStrategy import PpoStrategy
from .RciStrategy import RciStrategy
from .RsiCrossoverStrategy import RsiCrossoverStrategy
from .RsiReversalStrategy import RsiReversalStrategy
from .RsiStrategy import RsiStrategy
from .SmoothedAtrCrossoverStrategy import SmoothedAtrCrossoverStrategy
from .StochasticReversalStrategy import StochasticReversalStrategy
from .StochasticStrategy import StochasticStrategy
from .SuperTrendStrategy import SuperTrendStrategy
from .TripleMovingAverageStrategy import TripleMovingAverageStrategy
from .TsfCrossoverStrategy import TsfCrossoverStrategy
from .UltimateOscillatorStrategy import UltimateOscillatorStrategy
from .UpDownCrossoverStrategy import UpDownCrossoverStrategy
from .WilliamsRCrossoverStrategy import WilliamsRCrossoverStrategy
from .WilliamsRReversalStrategy import WilliamsRReversalStrategy
from .WilliamsRStrategy import WilliamsRStrategy

__all__ = [
    'AoStrategy',
    'ApoCrossoverStrategy',
    'AroonCrossoverStrategy',
    'AroonStrategy',
    'AtrChangeStrategy',
    'AtrCrossoverStrategy',
    'BollingerBandsStrategy',
    'BopStrategy',
    'CciStrategy',
    'CmoStrategy',
    'CorrelationCrossoverStrategy',
    'CorrelationMaCrossoverStrategy',
    'DmiStrategy',
    'GarmanKlassStrategy',
    'HeikinAshiStrategy',
    'HistoricalVolatilityStrategy',
    'IchimokuCloudStrategy',
    'KalmanFilterChangeStrategy',
    'KalmanFilterCrossoverStrategy',
    'MacdCustomStrategy',
    'MacdStrategy',
    'MamaStrategy',
    'MfiStrategy',
    'MidpointStrategy',
    'MlCrossoverStrategy',
    'MlStrategy',
    'MomentumCrossoverStrategy',
    'MomentumStrategy',
    'MovingAverageCrossoverStrategy',
    'MovingAverageDifferenceStrategy',
    'ParabolicSarStrategy',
    'PpoStrategy',
    'RciStrategy',
    'RsiCrossoverStrategy',
    'RsiReversalStrategy',
    'RsiStrategy',
    'SmoothedAtrCrossoverStrategy',
    'StochasticReversalStrategy',
    'StochasticStrategy',
    'SuperTrendStrategy',
    'TripleMovingAverageStrategy',
    'TsfCrossoverStrategy',
    'UltimateOscillatorStrategy',
    'UpDownCrossoverStrategy',
    'WilliamsRCrossoverStrategy',
    'WilliamsRReversalStrategy',
    'WilliamsRStrategy',
]
